#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <pthread.h>

#include "m3_SerialClient.h"

#include "insist.h"


m3_SerialClient::m3_SerialClient (char*switchname) : m3_TestClient (switchname)
{
  pthread_mutex_init (&mutex, 0);
}

static void*read_thread (void*p)
{
  m3_SerialClient*client = (m3_SerialClient*)p;

  insistf (client);
  insistf (client->num_commands > 0);
  insistf (client->commands);

  for (int i = 0; i < client->num_commands; i++)
  {
    m3_serial_command_t*c = client->commands + i;
    m3_int result;

    int r = client->read ((char*) &result, sizeof (result));
    
    if (r != sizeof (result))
      return 0;
    
    c->result = htonl (result);

    if (c->rxlen)
    {
      insistf (c->rxbuf);

      r = client->read (c->rxbuf, c->rxlen);
      
      if (r != c->rxlen)
      {
	client->printFormat ("read failed");
	client->close ();
	return 0;
      }
    }
  }
  return (void*) 1;
  exception: return 0;
}

int m3_SerialClient::write_thread()
{
  insist (this);
  insist (num_commands > 0);
  insist (commands);

  for (int i = 0; i < num_commands; i++)
  {
    m3_serial_command_t*c = commands + i;

    insist (c->txlen >= 0 && c->txlen <= M3_TEST_MTU - sizeof (m3_test_message_t) - sizeof (m3_int) * 6);
    insist (c->rxlen >= 0 && c->rxlen <= M3_TEST_MTU - sizeof (m3_test_message_t) - sizeof (m3_int) * 6);
    insist (c->txbuf || !c->txlen);
    insist (c->rxbuf || !c->rxlen);
  
    m3_test_serial_message_t m;
  
    m.tp.type = htonl (M3_TEST_STREAM);
    m.tp.xbar = htonl (0);

    m.port = htonl (c->port);
    m.command = htonl (c->command);
    m.num_commands = htonl (num_commands);
    m.txlen = htonl (c->txlen);
    m.rxlen = htonl (c->rxlen);
  
    if (c->txlen) memcpy (m.payload, c->txbuf, c->txlen);
  
    int r = send ((char*) &m, sizeof (m3_test_message_t) + sizeof (m3_int) * 6 + c->txlen);    
    if (!r) return 0;
  }
  return 1;
  exception: return 0;
}


int m3_SerialClient::run (int num_commands, m3_serial_command_t*commands)
{
  pthread_t thread;
  void*value;
  int r;
  
  insist (this);
  insist (num_commands > 0);
  insist (commands);

  if (fd <= 0 && ipAddress)
    open (ipAddress);
  
  this->num_commands = num_commands;
  this->commands = commands;
    
  pthread_create (&thread, 0, read_thread, (void*) this);
  if (!(r = write_thread ()))
    printFormat ("write thread failed");
  
  
  pthread_join (thread, &value);

   /*
  if (!close ())
  {
    printFormat ("close failed");
    return 0;
  }
   */

  return r && value != 0;
  exception: return 0;
}

m3_SerialClient::~m3_SerialClient ()
{
  pthread_mutex_destroy (&mutex);
  close ();
}

int m3_SerialClient::close ()
{
  insist (this);
  
  if (fd)
    ::close (fd);
  
  fd = 0;

  return 1;
  exception: return 0;
}
